/*
 * Khoros: $Id$
 */

/*
 * $Log$
 */

/* gdbmdefs.h - The include file for dbm.  Defines structure and constants. */

/*  This file is part of KDBM, the GNU data base manager, by Philip A. Nelson.
    Copyright (C) 1990, 1991, 1993  Free Software Foundation, Inc.

    KDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.

    KDBM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KDBM; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@cs.wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226
*************************************************************************/


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Public kdbm header
   >>>>
   >>>>    Written By: You may contact the author by:
   >>>>		         e-mail:  phil@cs.wwu.edu
   >>>>	         	us-mail:  Philip A. Nelson
   >>>>				  Computer Science Department
   >>>>				  Western Washington University
   >>>>          Date: 
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _klibdb_dbm_h_
#define _klibdb_dbm_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/


	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/* Parameters to kdbm_setopt, specifing the type of operation to perform. */
#define	 KDBM_CACHESIZE	1	/* Set the cache size. */

/* The magic number */
#define KDBM_MAGIC_NUMBER_OLD 0x13579ace
#define KDBM_MAGIC_NUMBER 0x13579acf


/* In freeing blocks, we will ignore any blocks smaller (and equal) to
   IGNORE_SIZE number of bytes. */
#define IGNORE_SIZE 4

/* The number of key bytes kept in a hash bucket. */
#define SMALL    4

/* The number of bucket_avail entries in a hash bucket. */
#define BUCKET_AVAIL 6

/* The size of the bucket cache. */
#define DEFAULT_CACHESIZE  100

        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/
 
/* The available file space is stored in an "avail" table.  The one with
   most activity is contained in the file header. (See below.)  When that
   one filles up, it is split in half and half is pushed on an "avail
   stack."  When the active avail table is empty and the "avail stack" is
   not empty, the top of the stack is popped into the active avail table. */

/* the number of ints in a header */
#define KDBM_HEADER_NUM	13

/* The following structure is the element of the avaliable table.  */
typedef struct {
  	long   av_size;		/* The size of the available block. */
	long   av_adr;		/* The file address of the available block. */
      } avail_elem;

/* This is the actual table. The in-memory images of the avail blocks are
   allocated by malloc using a calculated size.  */
typedef struct {
	int   size;		/* The number of avail elements in the table.*/
	int   count;		/* The number of entries in the table. */
	long  next_block;	/* The file address of the next avail block. */
	avail_elem av_table[1]; /* The table.  Make it look like an array.  */
      } avail_block;

/* The dbm file header keeps track of the current location of the hash
   directory and the free space in the file.  */

typedef struct {
	unsigned char  machtype; /* add in the machine architecture type */
	int   header_magic;  /* 0x13579ace to make sure the header is good. */
	int   block_size;    /* The  optimal i/o blocksize from stat. */
	int   dir_size;	     /* Size in ints of the table.  */
	int   dir_bits;	     /* The number of address bits used in the table.*/
	int   bucket_size;   /* Size in bytes of a hash bucket struct. */
	int   bucket_elems;  /* Number of elements in a hash bucket. */
	long  next_block;    /* The next unallocated block address. */
	long  dir;	     /* File address of hash directory table. */
	avail_block avail;   /* This must be last because of the psuedo
				array in avail.  This avail grows to fill
				the entire block. */
      }  kdbm_header;


/* The dbm hash bucket element contains the full 31 bit hash value, the
   "pointer" to the key and data (stored together) with their sizes.  It also
   has a small part of the actual key value.  It is used to verify the first
   part of the key has the correct value without having to read the actual
   key. */

typedef struct {
	int   hash_value;	/* The complete 31 bit value. */
	int   key_size;		/* Size of key data in the file. */
	int   data_size;	/* Size of associated data in the file. */
	long  data_pointer;	/* The file address of the key record. The
				   data record directly follows the key.  */
	char  key_start[SMALL];	/* Up to the first SMALL bytes of the key.  */
      } bucket_element;


/* A bucket is a small hash table.  This one consists of a number of
   bucket elements plus some bookkeeping fields.  The number of elements
   depends on the optimum blocksize for the storage device and on a
   parameter given at file creation time.  This bucket takes one block.
   When one of these tables gets full, it is split into two hash buckets.
   The contents are split between them by the use of the first few bits
   of the 31 bit hash function.  The location in a bucket is the hash
   value modulo the size of the bucket.  The in-memory images of the
   buckets are allocated by malloc using a calculated size depending of
   the file system buffer size.  To speed up write, each bucket will have
   BUCKET_AVAIL avail elements with the bucket. */

typedef struct {
        int   av_count;		   /* The number of bucket_avail entries. */
	int   bucket_bits;	   /* The number of bits used to get here. */
	int   count;		   /* The number of element buckets full. */
        avail_elem bucket_avail[BUCKET_AVAIL];  /* Distributed avail. */
	bucket_element h_table[1]; /* The table.  Make it look like an array.*/
      } hash_bucket;

/* We want to keep from reading buckets as much as possible.  The following is
   to implement a bucket cache.  When full, buckets will be dropped in a
   least recently read from disk order.  */

/* To speed up fetching and "sequential" access, we need to implement a
   data cache for key/data pairs read from the file.  To find a key, we
   must exactly match the key from the file.  To reduce overhead, the
   data will be read at the same time.  Both key and data will be stored
   in a data cache.  Each bucket cached will have a one element data
   cache.  */

typedef struct {
        int   hash;
	int   data_size;
	int   key_size;
	int   file_pos;
	int   seek_pos;
	int   elem_loc;
	char  *key;
	char  *data;
      }  data_cache_elem;

typedef struct {
        hash_bucket *   ca_bucket;
	long            ca_adr;
	data_cache_elem ca_data;
	int		ca_changed;   /* Data in the bucket changed. */
      } cache_elem;



/* This final structure contains all main memory based information for
   a kdbm file.  This allows multiple kdbm files to be opened at the same
   time by one program. */

struct _kdbm {
	/* Global variables and pointers to dynamic variables used by kdbm.  */

  	/* The file name. */
	char *name;

	/* The reader/writer status. */
	int rdonly;

	/* The kdbm file descriptor which is set in kdbm_open.  */
	int  desc;

	/* The file header holds information about the database. */
	kdbm_header *header;

	/* The hash table directory from extendible hashing.  See Fagin et al, 
	   ACM Trans on Database Systems, Vol 4, No 3. Sept 1979, 315-344 */
	long   *dir;

	/* The bucket cache. */
	cache_elem *bucket_cache;
	int cache_size;
	int last_read;

	/* Points to the current hash bucket in the cache. */
	hash_bucket *bucket;

	/* The directory entry used to get the current hash bucket. */
	int  bucket_dir;

	/* Pointer to the current bucket's cache entry. */
	cache_elem *cache_entry;


	/* Bookkeeping of things that need to be written back at the
	   end of an update. */
	char  header_changed;
	char  directory_changed;
	char  bucket_changed;
	char  second_changed;

	/* The last key got with the firstkey/nextkey */
	kdatum keyptr;

	int   size_header;
	int   size_avail_block;
	int   size_avail_elem;
	int   mach_dir_size;
	long  mach_dir;
};


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

#endif /* _klibdb_dbm_h_ */
/* Don't add after this point */
